@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2018 DreamWorks Animation LLC
+// Copyright (c) DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -159,7 +159,7 @@ public:
     {
         ValueType sum = 0.0;
         for (int n = 0, s = int(mStencil.size()); n < s; ++n) sum += mStencil[n];
-        return sum / mStencil.size();
+        return sum / ValueType(mStencil.size());
     }
 
     /// @brief Return the smallest value in the stencil buffer.
@@ -334,9 +334,15 @@ public:
     ///     + v100 u(1-v)(1-w)     + v101 u(1-v)w     + v110 uv(1-w)     + v111 uvw
     inline ValueType interpolation(const math::Vec3<ValueType>& xyz) const
     {
-        const ValueType u = xyz[0] - BaseType::mCenter[0]; assert(u>=0 && u<=1);
-        const ValueType v = xyz[1] - BaseType::mCenter[1]; assert(v>=0 && v<=1);
-        const ValueType w = xyz[2] - BaseType::mCenter[2]; assert(w>=0 && w<=1);
+        OPENVDB_NO_TYPE_CONVERSION_WARNING_BEGIN
+        const ValueType u = xyz[0] - BaseType::mCenter[0];
+        const ValueType v = xyz[1] - BaseType::mCenter[1];
+        const ValueType w = xyz[2] - BaseType::mCenter[2];
+        OPENVDB_NO_TYPE_CONVERSION_WARNING_END
+
+        assert(u>=0 && u<=1);
+        assert(v>=0 && v<=1);
+        assert(w>=0 && w<=1);
 
         ValueType V = BaseType::template getValue<0,0,0>();
         ValueType A = static_cast<ValueType>(V + (BaseType::template getValue<0,0,1>() - V) * w);
@@ -362,9 +368,15 @@ public:
     ///     + v100 u(1-v)(1-w)     + v101 u(1-v)w     + v110 uv(1-w)     + v111 uvw
     inline math::Vec3<ValueType> gradient(const math::Vec3<ValueType>& xyz) const
     {
-        const ValueType u = xyz[0] - BaseType::mCenter[0]; assert(u>=0 && u<=1);
-        const ValueType v = xyz[1] - BaseType::mCenter[1]; assert(v>=0 && v<=1);
-        const ValueType w = xyz[2] - BaseType::mCenter[2]; assert(w>=0 && w<=1);
+        OPENVDB_NO_TYPE_CONVERSION_WARNING_BEGIN
+        const ValueType u = xyz[0] - BaseType::mCenter[0];
+        const ValueType v = xyz[1] - BaseType::mCenter[1];
+        const ValueType w = xyz[2] - BaseType::mCenter[2];
+        OPENVDB_NO_TYPE_CONVERSION_WARNING_END
+
+        assert(u>=0 && u<=1);
+        assert(v>=0 && v<=1);
+        assert(w>=0 && w<=1);
 
         ValueType D[4]={BaseType::template getValue<0,0,1>()-BaseType::template getValue<0,0,0>(),
                         BaseType::template getValue<0,1,1>()-BaseType::template getValue<0,1,0>(),
@@ -1307,17 +1319,20 @@ public:
     {
         const Coord& ijk = BaseType::getCenterCoord();
         const ValueType d = ValueType(mStencil[0] * 0.5 * mInvDx2); // distance in voxels / (2dx^2)
-        return math::Vec3<ValueType>(ijk[0] - d*(mStencil[2] - mStencil[1]),
-                                     ijk[1] - d*(mStencil[4] - mStencil[3]),
-                                     ijk[2] - d*(mStencil[6] - mStencil[5]));
+        OPENVDB_NO_TYPE_CONVERSION_WARNING_BEGIN
+        const auto value = math::Vec3<ValueType>(   ijk[0] - d*(mStencil[2] - mStencil[1]),
+                                                    ijk[1] - d*(mStencil[4] - mStencil[3]),
+                                                    ijk[2] - d*(mStencil[6] - mStencil[5]));
+        OPENVDB_NO_TYPE_CONVERSION_WARNING_END
+        return value;
     }
 
     /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
     unsigned int pos() const { return GradPt<i,j,k>::idx; }
-    
+
 private:
-    
+
     inline void init(const Coord& ijk)
     {
         BaseType::template setValue<-1, 0, 0>(mCache.getValue(ijk.offsetBy(-1, 0, 0)));
@@ -1329,7 +1344,7 @@ private:
         BaseType::template setValue< 0, 0,-1>(mCache.getValue(ijk.offsetBy( 0, 0,-1)));
         BaseType::template setValue< 0, 0, 1>(mCache.getValue(ijk.offsetBy( 0, 0, 1)));
     }
-    
+
     template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
@@ -1692,6 +1707,6 @@ private:
 
 #endif // OPENVDB_MATH_STENCILS_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2018 DreamWorks Animation LLC
+// Copyright (c) DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
